'''
	Auteur : Pierre Aubert
	Mail : aubertp7@gmail.com
	Licence : CeCILL-C
'''

'''{
First, we have to import tools to make the installation of our module :
}'''

from setuptools import setup
from setuptools import Extension
from setuptools import find_packages
import sys
import os
from platform import system
import subprocess

import numpy as np

'''{
Check is \textbf{Cython} package is installed. The \textbf{Cython} package makes the \textbf{python} extensions for wrappers.
}'''

try:
	from Cython.Distutils import build_ext
except ImportError:
	use_cython = False
	print("Cython not found")
	raise Exception('Please install Cython on your system')
else:
	use_cython = True

'''{
Some informations to be sure that we will not lose them is the file size increases :
}'''

NAME = 'hadamardpython'
VERSION = '1.0.0'
AUTHOR = 'Asterics developers'
AUTHOR_EMAIL = 'pierre.aubert@lapp.in2p3.fr'
URL = ''
DESCRIPTION = 'Asterics HPC hadamard python module'
LICENSE = 'Cecil-C'

'''{
Some C++ flags correction, because the \textbf{python} expects C sources and not C++ ones : 
}'''

# Remove the "-Wstrict-prototypes" compiler option, which isn't valid for C++.
import distutils.sysconfig
cfg_vars = distutils.sysconfig.get_config_vars()
for key, value in cfg_vars.items():
	if type(value) == str:
		value = value.replace("-Wstrict-prototypes", "")
		value = value.replace("-DNDEBUG", "")
		cfg_vars[key] = value

'''{
Since, we are compiling a function which has to be fast, we have to add extra compilation arguments such as in the C++ version :
}'''

extra_compile_args = ['-Werror', '-march=native',  '-mtune=native', '-ftree-vectorize', '-mavx2', '-O3', '-DVECTOR_ALIGNEMENT=32', '-g']

'''{
Now, we create the extension with our file sources and some cmake macro to handle the files positions :
}'''

packageName = 'hadamardpython'
ext_modules = [
	Extension(packageName, ['@CMAKE_CURRENT_SOURCE_DIR@/hadamardpython.cpp',
				'@CMAKE_CURRENT_SOURCE_DIR@/hadamard_intrinsics_pitch.cpp',
				'@CMAKE_CURRENT_SOURCE_DIR@/hadamardWrapper.cpp'
	],
	libraries=[],
	library_dirs=[],
	runtime_library_dirs=[],
	extra_link_args=[],
	extra_compile_args=extra_compile_args,

	include_dirs=['@CMAKE_CURRENT_SOURCE_DIR@/',
			np.get_include()]
	)
]

'''{
Finally, we create the module and cacth exeptions if it failed :
}'''

try:
	setup(name = NAME,
		version=VERSION,
		ext_modules=ext_modules,
		description=DESCRIPTION,
		install_requires=['numpy', 'cython'],
		author=AUTHOR,
		author_email=AUTHOR_EMAIL,
		license=LICENSE,
		url=URL,
		classifiers=[
			'Intended Audience :: Science/Research',
			'License :: OSI Approved ::Cecil-C',
			'Operating System :: OS Independent',
			'Programming Language :: Python :: 3',
			'Topic :: Scientific/Engineering :: Astronomy',
			'Development Status :: 3 - Alpha'],
	)
except Exception as e:
	print(str(e))
	sys.exit(-1)



