'''
	Auteur : Pierre Aubert
	Mail : aubertp7@gmail.com
	Licence : CeCILL-C
'''

'''{
We need also to import several packages :

\begin{itemize}
	\item \textbf{sys} : to make an output compatible with C++ performances output
	\item \textbf{numpy} : to deal with arrays
	\item \textbf{astericshpc} : to allocate arrays and do the performance test
\end{itemize}
}'''

import sys
import numpy as np
import astericshpc

'''{
The function to evaluate performances is built the same way such as the C++ one :
}'''

def getTimeFunctionSize(nbRepetition, nbElement):
	matX = np.asarray(np.random.random((nbElement, nbElement)), dtype=np.float32)
	matY = np.asarray(np.random.random((nbElement, nbElement)), dtype=np.float32)
	matRes = np.zeros((nbElement,nbElement), dtype=np.float32)
	
	timeBegin = astericshpc.rdtsc()
	for i in range(0, nbRepetition):
		np.matmul(matX, matY, matRes)
	
	timeEnd = astericshpc.rdtsc()
	elapsedTime = float(timeEnd - timeBegin)/float(nbRepetition)
	elapsedTimePerElement = elapsedTime/float(nbElement*nbElement)
	print("nbElement =",nbElement,", elapsedTimePerElement =",elapsedTimePerElement,"cy/el",", elapsedTime =",elapsedTime,"cy")
	print(str(nbElement) + "\t" + str(elapsedTimePerElement) + "\t" + str(elapsedTime),file=sys.stderr)

'''{
Then, we have a function to make all the points with a list of sizes :
}'''

def makeElapsedTimeValue(listSize, nbRepetition):
	for val in listSize:
		getTimeFunctionSize(nbRepetition, val)

'''{
Finally, we call the performances tests only if this script is executed as a main file and not if it is included by an other file :
}'''

if __name__ == "__main__":
	listSize = [	10,
			16,
			24,
			32,
			40,
			56,
			80,
			90,
			104]
	makeElapsedTimeValue(listSize, 1000000)




